/***********************************************************************
 *  $RCSfile: PageFooter.ascx.cs,v $
 * 
 *  Copyright (C) 2006 Novell, Inc.
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Author: Mike Lasky (mlasky@novell.com)
 * 
 ***********************************************************************/
namespace Novell.iFolderWeb.Admin
{
	using System;
	using System.Data;
	using System.Drawing;
	using System.Resources;
	using System.Web;
	using System.Web.UI;
	using System.Web.UI.WebControls;
	using System.Web.UI.HtmlControls;

	/// <summary>
	///		Summary description for PageFooter.
	/// </summary>
	public class PageFooter : System.Web.UI.UserControl
	{
		#region Class Members

		/// <summary>
		/// Resource Manager
		/// </summary>
		private ResourceManager rm;

		/// <summary>
		/// Web Controls
		/// </summary>
		protected ImageButton PageFirstButton;

		/// <summary>
		/// Web Controls
		/// </summary>
		protected ImageButton PagePreviousButton;

		/// <summary>
		/// Web Controls
		/// </summary>
		protected ImageButton PageNextButton;

		/// <summary>
		/// Web Controls
		/// </summary>
		protected ImageButton PageLastButton;

		/// <summary>
		/// Web Controls
		/// </summary>
		protected System.Web.UI.WebControls.Image PageFirstDisabledButton;

		/// <summary>
		/// Web Controls
		/// </summary>
		protected System.Web.UI.WebControls.Image PagePreviousDisabledButton;

		/// <summary>
		/// Web Controls
		/// </summary>
		protected System.Web.UI.WebControls.Image PageNextDisabledButton;

		/// <summary>
		/// Web Controls
		/// </summary>
		protected System.Web.UI.WebControls.Image PageLastDisabledButton;

		/// <summary>
		/// Web Controls
		/// </summary>
		protected Label PageText;

		/// <summary>
		/// Event that notifies the consumer that the PageFirstButton was clicked.
		/// </summary>
		public event ImageClickEventHandler PageFirstClick = null;

		/// <summary>
		/// Event that notifies the consumer that the PagePreviousButton was clicked.
		/// </summary>
		public event ImageClickEventHandler PagePreviousClick = null;

		/// <summary>
		/// Event that notifies the consumer that the PageNextButton was clicked.
		/// </summary>
		public event ImageClickEventHandler PageNextClick = null;

		/// <summary>
		/// Event that notifies the consumer that the PageLastButton was clicked.
		/// </summary>
		public event ImageClickEventHandler PageLastClick = null;

		#endregion

		#region Private Methods

		/// <summary>
		/// Gets the number of visible items in the specified DataGrid.
		/// </summary>
		/// <param name="dataGrid">DataGrid containing members.</param>
		/// <returns>The number of visible items in the DataGrid.</returns>
		private int GetVisibleItems( DataGrid dataGrid )
		{
			int count = 0;
			foreach( DataRow dr in ( dataGrid.DataSource as DataView ).Table.Rows )
			{
				if ( ( bool )dr[ 0 ] )
				{
					++count;
				}
			}

			return count;
		}

		/// <summary>
		/// Page_Load()
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void Page_Load(object sender, System.EventArgs e)
		{
			// localization
			rm = Application[ "RM" ] as ResourceManager;

			if ( !IsPostBack )
			{
				PageFirstButton.Attributes.Add( "title", GetString( "FIRSTPAGE" ) );
				PagePreviousButton.Attributes.Add( "title", GetString( "PREVIOUSPAGE" ) );
				PageNextButton.Attributes.Add( "title", GetString( "NEXTPAGE" ) );
				PageLastButton.Attributes.Add( "title", GetString( "LASTPAGE" ) );
				PageFirstDisabledButton.Attributes.Add( "title", GetString( "FIRSTPAGE" ) );
				PagePreviousDisabledButton.Attributes.Add( "title", GetString( "PREVIOUSPAGE" ) );
				PageNextDisabledButton.Attributes.Add( "title", GetString( "NEXTPAGE" ) );
				PageLastDisabledButton.Attributes.Add( "title", GetString( "LASTPAGE" ) );
			}
		}

		#endregion

		#region Protected Methods

		/// <summary>
		/// Event handler that gets called when the PageFirstButton is clicked.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		protected void OnFirstPageClicked( Object sender, ImageClickEventArgs e )
		{
			if ( PageFirstClick != null )
			{
				PageFirstClick( this, e );
			}
		}

		/// <summary>
		/// Event handler that gets called when the PageLastButton is clicked.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		protected void OnLastPageClicked( Object sender, ImageClickEventArgs e )
		{
			if ( PageLastClick != null )
			{
				PageLastClick( this, e );
			}
		}

		/// <summary>
		/// Event handler that gets called when the PageNextButton is clicked.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		protected void OnNextPageClicked( Object sender, ImageClickEventArgs e )
		{
			if ( PageNextClick != null )
			{
				PageNextClick( this, e );
			}
		}

		/// <summary>
		/// Event handler that gets called when the PagePreviousButton is clicked.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		protected void OnPreviousPageClicked( Object sender, ImageClickEventArgs e )
		{
			if ( PagePreviousClick != null )
			{
				PagePreviousClick( this, e );
			}
		}

		/// <summary>
		/// Get a Localized String
		/// </summary>
		/// <param name="key">Key to the localized string.</param>
		/// <returns>Localized string.</returns>
		protected string GetString( string key )
		{
			string s = rm.GetString( key );
			return s;
		}

		#endregion

		#region Public Methods

		/// <summary>
		/// Enables or disables the search navigation buttons depending on the display data index.
		/// </summary>
		/// <param name="dataGrid"></param>
		/// <param name="currentItemOffset"></param>
		/// <param name="totalItems"></param>
		/// <param name="multipleItemType"></param>
		/// <param name="singleItemType"></param>
		public void SetPageButtonState( DataGrid dataGrid, int currentItemOffset, int totalItems, string multipleItemType, string singleItemType )
		{
			int userCount = GetVisibleItems( dataGrid );

			PageText.Text = String.Format( 
				GetString( "PAGENUMBERFORMAT" ),
				( userCount > 0 ) ? currentItemOffset + 1 : 0, 
				currentItemOffset + userCount,
				totalItems,
				( totalItems == 1 ) ? singleItemType : multipleItemType );			

			if ( currentItemOffset == 0 )
			{
				PageFirstButton.Visible = false;
				PagePreviousButton.Visible = false;
				PageFirstDisabledButton.Visible = true;
				PagePreviousDisabledButton.Visible = true;
			}
			else
			{
				PageFirstButton.Visible = true;
				PagePreviousButton.Visible = true;
				PageFirstDisabledButton.Visible = false;
				PagePreviousDisabledButton.Visible = false;
			}

			if ( ( currentItemOffset + dataGrid.PageSize ) >= totalItems )
			{
				PageNextButton.Visible = false;
				PageLastButton.Visible = false;
				PageNextDisabledButton.Visible = true;
				PageLastDisabledButton.Visible = true;
			}
			else
			{
				PageNextButton.Visible = true;
				PageLastButton.Visible = true;
				PageNextDisabledButton.Visible = false;
				PageLastDisabledButton.Visible = false;
			}
		}

		#endregion

		#region Web Form Designer generated code

		/// <summary>
		/// OnInit()
		/// </summary>
		/// <param name="e"></param>
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		///		Required method for Designer support - do not modify
		///		the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			PageFirstButton.Click += new ImageClickEventHandler( OnFirstPageClicked );
			PagePreviousButton.Click += new ImageClickEventHandler( OnPreviousPageClicked );
			PageNextButton.Click += new ImageClickEventHandler( OnNextPageClicked );
			PageLastButton.Click += new ImageClickEventHandler( OnLastPageClicked );

			this.Load += new System.EventHandler(this.Page_Load);
		}

		#endregion
	}
}
